#include <windows.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include <math.h>

#define SEGMENTS   50
#define WAVES      2
#define AMPLITUDE  0.1
#define WIDTH      1.0
#define HEIGHT     0.7
#define CHECKS     16

const GLfloat angle_inc = (GLfloat)M_PI * 2.0 * (GLfloat)WAVES / (GLfloat)SEGMENTS;
const GLfloat x_inc = WIDTH / (GLfloat)SEGMENTS;
const GLfloat y_inc = 0.35 / GLfloat(SEGMENTS / 2);
const GLfloat c_inc = 2.0 / (GLfloat)CHECKS;

UINT timer;
HINSTANCE MainInstance;
int offs = 0;

void setDCPixelFormat(HDC hdc)
{
  static PIXELFORMATDESCRIPTOR pfd =
  {
    sizeof(PIXELFORMATDESCRIPTOR),          // sizeof this structure
    1,                                      // version number
    PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL
    | PFD_DOUBLEBUFFER,                     // flags
    PFD_TYPE_RGBA,                          // RGBA pixel values
    16,                                     // 24-bit color
    0, 0, 0, 0, 0, 0,                       // don't care about these
    0, 0,                                   // no alpha buffer
    0, 0, 0, 0, 0,                          // no accumulation buffer
    16,                                     // 32-bit depth buffer
    0,                                      // no stencil buffer
    0,                                      // no auxiliary buffers
    PFD_MAIN_PLANE,                         // layer type
    0,                                      // reserved(must be 0)
    0, 0, 0                                 // no layer masks
  };
  int pixelFormat;

  pixelFormat = ChoosePixelFormat(hdc, &pfd);
  SetPixelFormat(hdc, pixelFormat, &pfd);
  DescribePixelFormat(hdc, pixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd);
}

void InitializeGL()
{
  GLfloat ambient[] = { 0.0, 0.0, 0.0, 1.0 };
  GLfloat diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
  GLfloat position[] = { 0.0, 0.0, 5.0, 0.0 };
  GLfloat lmodel_ambient[] = { 0.4, 0.4, 0.4, 1.0 };
  GLfloat local_view[] = { 0.0 };

  glFrontFace(GL_CW);
  glCullFace(GL_BACK);
  glEnable(GL_CULL_FACE);

  glDepthFunc(GL_LEQUAL);
  glEnable(GL_DEPTH_TEST);
  glClearColor(0.0, 0.0, 0.0, 0.0);

  glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
  glLightfv(GL_LIGHT0, GL_POSITION, position);
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
  glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, local_view);

  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
}

void DrawFlag(int o)
{
  GLfloat arg = M_PI * o / 180, n_x1, n_z1, n_x2, n_z2, z1, z2, mod;
  GLfloat y1, y2 = 0;

  z2 = sin(arg) * (GLfloat)AMPLITUDE;
  n_x2 = -cos(arg);
  n_z2 = 1;
  mod = (GLfloat)sqrt(n_x2 * n_x2 + 1);
  n_x2 /= mod;
  n_z2 /= mod;

  for (int i = 0; i < SEGMENTS; i++)
  {
    arg += angle_inc;
    z1 = sin(arg) * (GLfloat)AMPLITUDE;
    n_x1 = -cos(arg);
    n_z1 = 1;
    mod = (GLfloat)sqrt(n_x1 * n_x1 + 1);
    n_x1 /= mod;
    n_z1 /= mod;
    if (i <= SEGMENTS / 2)
    {
      glBegin(GL_QUADS);
        glNormal3f(n_x2, 0.0, n_z2);
        glVertex3f(i * x_inc, (GLfloat)HEIGHT / 2, z2 + AMPLITUDE * 2);
        glNormal3f(n_x1, 0.0, n_z1);
        glVertex3f((i + 1) * x_inc, (GLfloat)HEIGHT / 2, z1 + AMPLITUDE * 2);
        glVertex3f((i + 1) * x_inc, -(GLfloat)HEIGHT / 2, z1 + AMPLITUDE * 2);
        glNormal3f(n_x2, 0.0, n_z2);
        glVertex3f(i * x_inc, -(GLfloat)HEIGHT / 2, z2 + AMPLITUDE * 2);
      glEnd();
    }
    else
    {
      y1 = (i - SEGMENTS / 2) * y_inc;

      glBegin(GL_QUADS);
        glNormal3f(n_x2, 0.0, n_z2);
        glVertex3f(i * x_inc, (GLfloat)HEIGHT / 2, z2 + AMPLITUDE * 2);
        glNormal3f(n_x1, 0.0, n_z1);
        glVertex3f((i + 1) * x_inc, (GLfloat)HEIGHT / 2, z1 + AMPLITUDE * 2);
        glVertex3f((i + 1) * x_inc, y1, z1 + AMPLITUDE * 2);
        glNormal3f(n_x2, 0.0, n_z2);
        glVertex3f(i * x_inc, y2, z2 + AMPLITUDE * 2);

        glVertex3f(i * x_inc, -y2, z2 + AMPLITUDE * 2);
        glNormal3f(n_x1, 0.0, n_z1);
        glVertex3f((i + 1) * x_inc, -y1, z1 + AMPLITUDE * 2);
        glVertex3f((i + 1) * x_inc, -(GLfloat)HEIGHT / 2, z1 + AMPLITUDE * 2);
        glNormal3f(n_x2, 0.0, n_z2);
        glVertex3f(i * x_inc, -(GLfloat)HEIGHT / 2, z2 + AMPLITUDE * 2);
      glEnd();

      y2 = y1;
    }
    z2 = z1;
    n_x2 = n_x1;
    n_z2 = n_z1;
  }
}

void DrawChecks()
{
  GLfloat blue[] = { 0.2, 0.2, 1.0, 1.0 };
  GLfloat black[] = { 0.0, 0.0, 0.5, 1.0 };

  for (int j = 0; j < CHECKS; j++)
    for (int i = 0; i < CHECKS; i++)
    {
      if ((i + j) & 1)
        glMaterialfv(GL_FRONT, GL_DIFFUSE, blue);
      else
        glMaterialfv(GL_FRONT, GL_DIFFUSE, black);

      glBegin(GL_QUADS);
        glNormal3f(0.0, 0.0, 1.0);
        if ((i + j) & 1)
          glColor4fv(blue);
        else
          glColor4fv(black);
        glVertex3f(-1 + (i + 1) * c_inc, -1 + (j + 1) * c_inc, 0);
        glVertex3f(-1 + (i + 1) * c_inc, -1 + j * c_inc, 0);
        glVertex3f(-1 + i * c_inc, -1 + j * c_inc, 0);
        glVertex3f(-1 + i * c_inc, -1 + (j + 1) * c_inc, 0);
      glEnd();
    }
}

void drawScene(HDC hDC)
{
  GLfloat no_mat[] = { 0.0, 0.0, 0.0, 1.0 };
  GLfloat mat_diffuse[] = { 1.0, 0.2, 0.2, 0.9 };
  GLfloat no_shininess[] = { 0.0 };
  GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
  GLfloat high_shininess[] = { 100.0 };
  static GLfloat angle = 0;

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  glMaterialfv(GL_FRONT, GL_AMBIENT, no_mat);
  glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
  glMaterialfv(GL_FRONT, GL_SHININESS, high_shininess);
  glMaterialfv(GL_FRONT, GL_EMISSION, no_mat);
  glDisable(GL_LIGHTING);
  glPushMatrix();
  glTranslatef(0.0, -0.3, -1.7);
  glRotatef(-60, 1.0, 0.0, 0.0);
  glRotatef(angle * 180 / (2 * M_PI), 0.0, 0.0, 1.0);
  DrawChecks();
  glPopMatrix();
  glClear(GL_DEPTH_BUFFER_BIT);
  glEnable(GL_LIGHTING);

  glTranslatef(0.0, -0.07, -3.0);
  glRotatef(sin(angle) * 10, 0.0, 0.0, 1.0);
  glRotatef(-20, 1.0, 0.0, 0.0);
  glRotatef(10, 0.0, 1.0, 0.0);

  glMaterialfv(GL_FRONT, GL_AMBIENT, no_mat);
  glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
  glMaterialfv(GL_FRONT, GL_SPECULAR, no_mat);
  glMaterialfv(GL_FRONT, GL_SHININESS, no_shininess);
  glMaterialfv(GL_FRONT, GL_EMISSION, no_mat);
  DrawFlag(-offs);

  glRotatef(180, 0.0, 0.0, 1.0);
  DrawFlag(offs + 180);

  SwapBuffers(hDC);
  glFlush();

  angle += 0.03;
  if (angle > 2 * M_PI) angle -= 2 * M_PI;
}

LONG WINAPI wndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  static HDC hdc;
  static HGLRC hrc;
  PAINTSTRUCT ps;
  GLdouble aspect;
  GLsizei width, height;

  switch(msg)
  {
    case WM_CREATE:
                hdc = GetDC(hWnd);
                setDCPixelFormat(hdc);
                hrc = wglCreateContext(hdc);
        wglMakeCurrent(hdc, hrc);
      InitializeGL();
      timer = SetTimer(hWnd, 1, 50, NULL);
      return 0;

    case WM_SIZE:
      width = (GLsizei)LOWORD(lParam);
      height = (GLsizei)HIWORD(lParam);
      if (!width || !height) return 0;

      aspect = (GLdouble)width /(GLdouble)height;
      glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      gluPerspective(30.0, aspect, 1.0, 100.0);
      glViewport(0, 0, width, height);

      return 0;

    case WM_PAINT:
      BeginPaint(hWnd, &ps);
      drawScene(hdc);
      EndPaint(hWnd, &ps);
      return 0;

    case WM_DESTROY:
      wglMakeCurrent(NULL, NULL);
      wglDeleteContext(hrc);
      KillTimer(hWnd, timer);
      PostQuitMessage(0);
      return 0;

    case WM_TIMER:
      offs += 8;
      if (offs > 360)
        offs -= 360;
      InvalidateRect(hWnd, NULL, FALSE);
      return 0;
  }

  return DefWindowProc(hWnd, msg, wParam, lParam);
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR cmdLine, int cmdShow)
{
  static char appName [] = "Flag - by HackMaster";
  WNDCLASS wc;
  HWND hWnd;
  MSG msg;

  MainInstance = hInstance;

  wc.style = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc =(WNDPROC)wndProc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = hInstance;
  wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground =(HBRUSH)(COLOR_WINDOW + 1);
  wc.lpszMenuName = (LPCTSTR)0;
  wc.lpszClassName = appName;

  RegisterClass(&wc);

  hWnd = CreateWindow(appName, appName,
                      WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
                      CW_USEDEFAULT, CW_USEDEFAULT, 320,
                      240, HWND_DESKTOP, NULL, hInstance, NULL);

  ShowWindow(hWnd, cmdShow);
  UpdateWindow(hWnd);

  while(GetMessage(&msg, NULL, 0, 0))
  {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }

  return msg.wParam;
}
